@@ -10,44 +10,44 @@ use File::Basename;
 use XML::Writer;
 use IO::File;
 
-our $VERSION = '1.20';
+our $VERSION = '1.3';
 
+# Hash tables for radius events
 my %tags = ();
 my %event;
 my %start;
 my %stop;
 my %interim;
+
 my $interimUpdate;
-my $orphanDir = '/tmp';
-my $debug = 0;
 my @labels;
 my $writer;
 my $daysForOrphan = 1;
 my $labelref;
-my $purgeOrphan = 0;
-my $writeAllEvents = 0;
-my $xmlencoding = "utf-8";
-my $fname;
-my $outputDir = '/tmp';
+
 my $startDbm;
 my $interimDbm;
 my $stopDbm;
-my $controlDataStructure = 0;
 
+my $orphanDir = '/tmp';
+my $debug = 0;
+my $purgeOrphan = 0;
+my $writeAllEvents = 0;
+my $controlDataStructure = 0;
+my $outputDir = '/tmp';
+my $xmlencoding = "utf-8";
 
 
 #--------------------------------------------------
-#Construct the parser object
-
+#Constructor
 sub new {
 
 	my $this   = shift;
 	my $class  = ref($this) || $this;
 	my %params = @_;
-	$fname = "radius::XMLparser::new()";
 
 	#Load parameters if any
-	$debug = $params{DEBUG} if ($params{DEBUG});
+	$debug = $params{DEBUG} if $params{DEBUG};
 	$labelref = $params{LABELS} if $params{LABELS};
 	$purgeOrphan = $params{AUTOPURGE} if $params{AUTOPURGE};
 	$daysForOrphan = $params{DAYSFORORPHAN} if $params{DAYSFORORPHAN};
@@ -59,22 +59,21 @@ sub new {
 	@labels = @$labelref if $labelref;
 	
 	#Print supplied parameters
-	foreach my $key (keys %params){
-		print "$fname: CONF: params{$key} = ".$params{$key}."\n" if $debug;
+	if ($debug){
+		foreach my $key (keys %params){
+			print "CONF: params{$key} = ".$params{$key}."\n";
+		}	
 	}
 
-	$startDbm = "$orphanDir/orphan.start";
-	$interimDbm = "$orphanDir/orphan.interim";
-	$stopDbm = "$orphanDir/orphan.stop";
+	$startDbm = "$orphanDir/radius.orphan.start";
+	$interimDbm = "$orphanDir/radius.orphan.interim";
+	$stopDbm = "$orphanDir/radius.orphan.stop";
 
 	#Load previously stored hash if any
 	_loadHash();
 
-
 	my $self   = {};
-	#Amen !
 	bless $self => $class;
-
 	$self;
 }
 
@@ -82,12 +81,10 @@ sub new {
 
 #--------------------------------------------------
 #Open log file and parse each line. Group then all event by session ID
-
 sub group($$) {
 
 	my ($self, $logFileRef) = @_;
 	my $totalErrors = 0;
-	$fname = "RADIUS::XMLParser::group()";
 
 	#Logs files to analyze can be provided as an array ref
 	for my $log (@$logFileRef){
@@ -98,7 +95,7 @@ sub group($$) {
 
 		#Open log file to be parsed
 		open (LOG, $log) or croak "Cannot open file; File=$log; $!";
-		print "$fname: Will now parse file $log\n" if $debug;
+		print "Will now parse file $log\n" if $debug;
 
 		#Boolean that becomes true (1) when the first blank lines have been skipped.
 		my $begining_skipped = 0;
@@ -107,7 +104,6 @@ sub group($$) {
 		while ( <LOG> ) {
 
 			$processedLines++;
-
 			# Skip the begining of the log file if it only contains blank lines.
 			if (/^(\s)*$/ && !$begining_skipped) {
 				next;
@@ -122,54 +118,53 @@ sub group($$) {
 		}
 
 		#Print sum up
-		print "$fname: TOTAL ".$processedLines ." line(s) have been processed\n" if $debug;
-		print "$fname: TOTAL ".$errorLines ." error(s) found\n" if $debug;
-		print "$fname: TOTAL ".scalar(keys %stop)." stop event(s) found\n" if $debug;
-		print "$fname: TOTAL ".scalar(keys %start)." start event(s) found\n" if $debug;
+		print "TOTAL ".$processedLines ." line(s) have been processed\n" if $debug;
+		print "TOTAL ".$errorLines ." error(s) found\n" if $debug;
+		print "TOTAL ".scalar(keys %stop)." stop event(s) found\n" if $debug;
+		print "TOTAL ".scalar(keys %start)." start event(s) found\n" if $debug;
 
 		#Control data structure
 		if ($controlDataStructure){
 			
-			
 			my $stopControlFile = $stopDbm.".dbm";
 			my $startControlFile = $startDbm.".dbm";
 			my $interimControlFile = $interimDbm.".dbm";
 			
 			open (STOP, ">", $stopControlFile) or croak "Cannot open file; File=$stopControlFile; $!";
-			print "$fname: Printing out STOP dataStructure to $stopControlFile\n" if $debug;
+			print "Printing out STOP dataStructure to $stopControlFile\n" if $debug;
 			print STOP Dumper(%stop);
 			close(STOP);
 			open (START, ">", $startControlFile) or croak "Cannot open file; File=$startControlFile; $!";
-			print "$fname: Printing out START dataStructure to $startControlFile\n" if $debug;
+			print "Printing out START dataStructure to $startControlFile\n" if $debug;
 			print START Dumper(%start);
 			close(START);
 			open (INTERIM, ">", $interimControlFile) or croak "Cannot open file; File=$interimControlFile; $!";
-			print "$fname: Printing out INTERIM dataStructure to $interimControlFile\n" if $debug;
+			print "Printing out INTERIM dataStructure to $interimControlFile\n" if $debug;
 			print INTERIM Dumper(%interim);
 			close(INTERIM);
 			
 		}
 
 		#Store file into XML
-		print "$fname: Will now convert $log into xml\n" if $debug;
+		print "Will now convert $log into xml\n" if $debug;
 		$self->convert($log);
 		
-		$fname = "RADIUS::XMLParser::group()";
 		#Log has been parsed
 		close (LOG);
 
-		print "$fname: All done for file $log\n" if $debug;
+		print "All done for file $log\n" if $debug;
 
-		print "$fname: Reinitializing Stop hash table\n";
+		print "Reinitializing Stop hash table\n";
 		%stop = ();
 		$totalErrors += $errorLines;
 
-	} #Next log file
+	#Next log file
+	}
 
 	#Print sum up
-	print "$fname: All Done for all files!\n" if $debug;
-	print "$fname: Orphan Start is now ".scalar(keys %start)." items long\n" if $debug;
-	print "$fname: Orphan Interim is now ".scalar(keys %interim)." items long\n" if $debug;
+	print "All Done for all files!\n" if $debug;
+	print "Orphan Start is now ".scalar(keys %start)." items long\n" if $debug;
+	print "Orphan Interim is now ".scalar(keys %interim)." items long\n" if $debug;
 
 	return $totalErrors;
 
@@ -178,11 +173,9 @@ sub group($$) {
 
 #--------------------------------------------------
 #Convert Stop event hash reference to XML
-
 sub convert($$){
 
 	my ($self, $log) = @_;
-	$fname = "RADIUS::XMLParser::convert()";
 
 	#Initialize counter
 	my $stopevents = 0;
@@ -192,26 +185,29 @@ sub convert($$){
 	#Create output xml file	
 	$log = basename($log);
 	my $xml = $log;
-	if($log =~ m/\.log/){
+	if($log =~ m/\.log$/){
 		$xml =~ s/\.log/\.xml/g;
 	} else {
 		$xml = $log .".xml";
 	}
 	
-
 	#Create a new IO::File
-	my $output = IO::File->new(">$outputDir/$xml") or croak "Cannot open file $xml, $!";
-	print "$fname: Will now write XML content into $outputDir/$xml\n" if $debug;
+	my $output = IO::File->new(">$outputDir/$xml") or croak "Cannot open file $outputDir/$xml, $!";
+	print "Will now write XML content into $outputDir/$xml\n" if $debug;
 	
 	#Load XML:Writer
-	$writer = XML::Writer->new(OUTPUT => $output, ENCODING => $xmlencoding, DATA_MODE => 1, DATA_INDENT => 1) or croak "cannot create XML::Writer: $!";
+	$writer = XML::Writer->new(
+		OUTPUT => $output, 
+		ENCODING => $xmlencoding, 
+		DATA_MODE => 1, 
+		DATA_INDENT => 1
+	) or croak "cannot create XML::Writer: $!";
 
 	#Start writing
 	$writer->xmlDecl(uc($xmlencoding));
 
 	#Write a new SESSIONS tag
 	$writer->startTag("sessions");
-
 	
 	#For each provided Stop event
 	foreach my $sessionId (keys %stop){
@@ -248,7 +244,6 @@ sub convert($$){
 
 				#Open INTERIM tag
 				$writer->startTag("interim", "id"=>$event);
-				
 				$interimevents++;
 			
 				#Write content
@@ -275,7 +270,6 @@ sub convert($$){
 	}
 	
 	
-	
 	#ALTERNATIVE
 	#If User wants all events to be reported, let us process start event
 	if ($writeAllEvents){
@@ -305,7 +299,6 @@ sub convert($$){
 	
 					#Open INTERIM tag
 					$writer->startTag("interim", "id"=>$event);
-					
 					$interimevents++;
 				
 					#Write content
@@ -338,7 +331,6 @@ sub convert($$){
 		#If User wants all events to be reported, let us process interim event
 		for my $sessionId (keys %interim){
 			
-			
 			#Open a SESSION Tag
 			$writer->startTag("session", 'sessionId' => $sessionId);
 			my $newRef = $interim{$sessionId};
@@ -354,7 +346,6 @@ sub convert($$){
 
 				#Open INTERIM tag
 				$writer->startTag("interim", "id"=>$event);
-			
 				$interimevents++;
 			
 				#Write content
@@ -375,23 +366,17 @@ sub convert($$){
 			
 			#And delete orphan record
 			delete $interim{$sessionId};
-			
-			
 		}
 		
-		
-		
-		
 	}
 	
 	#Close SESSIONS tag
 	$writer->endTag("sessions");
 
 	#Print sum up
-	print "$fname: $stopevents Stop event(s) have been written\n" if $debug;
-	print "$fname: $startevents Start event(s) have been found and written\n" if $debug;
-	print "$fname: $interimevents Interims event(s) have been found and written\n" if $debug;
-
+	print "$stopevents Stop event(s) have been written\n" if $debug;
+	print "$startevents Start event(s) have been found and written\n" if $debug;
+	print "$interimevents Interims event(s) have been found and written\n" if $debug;
 
 	return 1;
 
@@ -400,7 +385,6 @@ sub convert($$){
 
 #--------------------------------------------------
 #Remove oldest keys from hash
-
 sub _removeStartOldEntries($){
 
 	my $hashref = shift;
@@ -412,7 +396,7 @@ sub _removeStartOldEntries($){
 	#Compute threshold in seconds
 	my $threshold = $daysForOrphan * 24 * 3600;
 
-	print "$fname: Remove Start orphan older than $daysForOrphan day(s)\n" if $debug;
+	print "Remove Start orphan older than $daysForOrphan day(s)\n" if $debug;
 
 	#Run through Start hash table
 	foreach my $sessionId (keys %hash){
@@ -421,7 +405,7 @@ sub _removeStartOldEntries($){
 
 		if (!$newHash{"Event-Timestamp"}){
 			#Delete records without date
-			print "$fname: Cannot find timestamp, delete entry\n" if $debug > 9;
+			print "Cannot find timestamp, delete entry\n" if $debug > 9;
 			delete $hash{$sessionId};
 			next;
 		}
@@ -432,7 +416,7 @@ sub _removeStartOldEntries($){
 
 		if ($delta > $threshold){
 			#Delete oldest records
-			print "$fname: delta is $delta - too old orphan\n" if $debug > 14;
+			print "delta is $delta - too old orphan\n" if $debug > 14;
 			delete $hash{$sessionId};
 			next;
 		}
@@ -446,7 +430,6 @@ sub _removeStartOldEntries($){
 
 #--------------------------------------------------
 #Remove oldest keys from hash
-
 sub _removeInterimOldEntries($){
 
         my $hashref = shift;
@@ -458,7 +441,7 @@ sub _removeInterimOldEntries($){
 		#Compute threshold in seconds
         my $threshold = $daysForOrphan * 24 * 3600;
 
-		print "$fname: Remove Interim orphan older than $daysForOrphan day(s)\n" if $debug;
+		print "Remove Interim orphan older than $daysForOrphan day(s)\n" if $debug;
 
 		#Run through Interim hash tables
         foreach my $sessionId (keys %hash){
@@ -470,7 +453,7 @@ sub _removeInterimOldEntries($){
 
                 	if (!$newNewHash{"Event-Timestamp"}){
 							#Delete records without date
-                        	print "$fname: Cannot find timestamp, delete entry\n" if $debug > 9;
+                        	print "Cannot find timestamp, delete entry\n" if $debug > 9;
                         	delete $newHash{$occurence};
                         	next;
                 	}
@@ -481,7 +464,7 @@ sub _removeInterimOldEntries($){
 
                 	if ($delta > $threshold){
 							#Delete oldest records
-                        	print "$fname: delta is $delta - too old orphan\n" if $debug > 14;
+                        	print "delta is $delta - too old orphan\n" if $debug > 14;
                         	delete $newHash{$occurence};
                         	next;
                 	}
@@ -501,19 +484,18 @@ sub _removeInterimOldEntries($){
 
 #--------------------------------------------------
 #Retrieve an orphan Start event based on sessionId
-
 sub _findInStartQueue($){
 
 	my ($sessionId) = @_;
 	my $eventref = $start{$sessionId};
 	if (scalar (keys %$eventref)){
 		#found Start event
-		print "$fname : retrieved Start event for session ID $sessionId\n" if $debug > 9;
+		print "retrieved Start event for session ID $sessionId\n" if $debug > 9;
 		#Remove start event from orphan hash
 		delete $start{$sessionId};
 	} else {
 		#not found
-		print "$fname : Orphan stop without start for session ID $sessionId\n" if $debug > 14;
+		print "Orphan stop without start for session ID $sessionId\n" if $debug > 14;
 	}
 
 	#Return hash reference of found Start event, undef otherwise
@@ -524,19 +506,18 @@ sub _findInStartQueue($){
 
 #--------------------------------------------------
 #Retrieve an orphan interim event based on sessionId
-
 sub _findInInterimQueue($){
 
 	my ($sessionId) = @_;
 	my $eventref = $interim{$sessionId};
 	if (scalar (keys %$eventref)){
 		#found Start event
-		print "$fname : retrieved Interim event for session ID $sessionId\n" if $debug > 9;
+		print "retrieved Interim event for session ID $sessionId\n" if $debug > 9;
 		#Remove interim event from orphan hash
 		delete $interim{$sessionId};
 	} else {
 		#not found
-		print "$fname : Orphan stop without interim for session ID $sessionId\n" if $debug > 14;
+		print "Orphan stop without interim for session ID $sessionId\n" if $debug > 14;
 	}
 
 	#Return hash reference of found Start event, undef otherwise
@@ -548,10 +529,8 @@ sub _findInInterimQueue($){
 
 #--------------------------------------------------
 #Convert a set of key value from a given hash ref into XML
-
 sub _writeEvent($){
 
-
 	my $ref = shift;
 	my %hash = %$ref;
 
@@ -563,7 +542,6 @@ sub _writeEvent($){
 		}
 		push (@labels, "File");
 	}
-	
 
 	#convert only the supplied label
 	for my $key (@labels){
@@ -578,18 +556,15 @@ sub _writeEvent($){
 			
 	}
 
-
 }
 
 
 #--------------------------------------------------
 #Read stored hash if file exists
-
 sub _loadHash() {
 
-
         #Load previously stored hashes
-        print "$fname: Loading stored data structures\n" if $debug;
+        print "Loading stored data structures\n" if $debug;
         my $startref;
         my $interimref;
 
@@ -598,9 +573,9 @@ sub _loadHash() {
                 $startref = lock_retrieve($startDbm) or croak "cannot open file $startDbm: $!";
 				$startref = _removeStartOldEntries($startref) if $purgeOrphan;
                 %start = %$startref;
-				print "$fname: Start hash is now ".scalar(keys %start)." items long\n"; 
+				print "Start hash is now ".scalar(keys %start)." items long\n"; 
         } else {
-				print "$fname: Start orphan hash is currently empty\n" if $debug;
+				print "Start orphan hash is currently empty\n" if $debug;
 				#Does not exist, so initialize a new one
                 %start = ();
         }
@@ -610,9 +585,9 @@ sub _loadHash() {
                 $interimref = lock_retrieve($interimDbm) or croak "cannot open file $interimDbm: $!";
 				$interimref = _removeInterimOldEntries($interimref) if $purgeOrphan;
                 %interim = %$interimref;
-				print "$fname: Interim hash is now ".scalar(keys %interim)." items long\n"; 
+				print "Interim hash is now ".scalar(keys %interim)." items long\n"; 
         } else {
-				print "$fname: Interim orphan hash is currently empty\n" if $debug;
+				print "Interim orphan hash is currently empty\n" if $debug;
 				#Does not exist, so initialize a new one
                 %interim = ();
         }
@@ -620,18 +595,14 @@ sub _loadHash() {
 }
 
 
-
-
 #--------------------------------------------------
 #Retrieve the highest numeric key from a given hash
-
 sub _largestKeyFromHash ($) {
 
 	my ($hash) = shift;
 	my ($key, @keys) = keys   %$hash;
 	my ($big, @vals) = values %$hash;
 
-
 	for (0 .. $#keys) {
 		if ($vals[$_] > $big) {
 			$big = $vals[$_];
@@ -641,24 +612,19 @@ sub _largestKeyFromHash ($) {
 
 	#Return highest key value
 	return $key
-
 }
 
 
-
 #--------------------------------------------------
 #Parse each line given as Input buffer
-
 sub _analyseRadiusLine($$$) {
-
 	
 	my ( $line, $lineNumber, $file ) = @_;
 	
 	#Radius Date Format (1st line)
 	#Should contain both MON and DAY (letter) And timestamp HH:MI:SS
 	if ($line =~ /^[A-Za-z]{3}.*[A-Za-z]{3}/ && $line =~ /[0-9]{2}[:][0-9]{2}[:][0-9]{2}/){
-     
-		print "$fname: New event, initialize hash\n" if $debug > 9;   	
+		print "New event, initialize hash\n" if $debug > 9;   	
 		%event = ();
 
 	#Empty line (end of session - Last line)
@@ -669,8 +635,7 @@ sub _analyseRadiusLine($$$) {
 		my $file = basename($file);
 
 		if ($val =~ /.*[S,s]tart.*/){
-
-			print "$fname: Add event to start hashtable\n" if $debug > 4;
+			print "Add event to start hashtable\n" if $debug > 4;
 			foreach my $key (keys %event){
 				#Store local start event to global Start events hash
 				$start{$sessionId}{$key}=$event{$key};
@@ -678,8 +643,7 @@ sub _analyseRadiusLine($$$) {
 			$start{$sessionId}{File}=$file;
 
 		} elsif ($val =~ /.*[S,s]top.*/){
-
-			print "$fname: Add event to Stop hashtable\n" if $debug > 4;
+			print "Add event to Stop hashtable\n" if $debug > 4;
 			foreach my $key (keys %event){
 				#Store local stop event to global Stop events hash
 				$stop{$sessionId}{$key}=$event{$key};
@@ -687,10 +651,9 @@ sub _analyseRadiusLine($$$) {
 			$stop{$sessionId}{File}=$file;
 
 		} elsif ($val =~ /.*[I,i]nterim/){
-
 			$interimUpdate = _largestKeyFromHash($interim{$sessionId});
 			$interimUpdate ++;
-			print "$fname: Add event to Interim hashtable\n" if $debug > 4;
+			print "Add event to Interim hashtable\n" if $debug > 4;
 			foreach my $key (keys %event){
 				#Store local interim event to global Interims events hash
 				$interim{$sessionId}{$interimUpdate}{$key}=$event{$key};
@@ -698,27 +661,25 @@ sub _analyseRadiusLine($$$) {
 			}
 		} else {
 			#Unmanaged event
-			print "$fname: unmanaged event, line $lineNumber\n" if $debug;
+			print "unmanaged event [$val], line $lineNumber\n" if $debug > 4;
 			return undef;
 		}
 		
-
 	#Between first and last line, we store any TAG/VALUE found
-	#2012-07-28 Allowing space character in value and numbers in key
 	} elsif ( my($tag,$val) = ( $line =~ m/^\t([0-9A-Za-z:-]+)\s+=\s+["]?([A-Za-z0-9=\\\.-\_\s]*)["]?.*\n/ ) ) {
 
 		if($tag){
 			$tags{$tag}++;
 			$event{$tag} = $val;
-			print "$fname: $lineNumber: $tag = ".$event{$tag}."\n" if $debug > 9;
+			print "$lineNumber: $tag = ".$event{$tag}."\n" if $debug > 9;
 		} else {
-			print "$fname: Unknown line $lineNumber, cannot find tag/value" if $debug;
+			print "Unknown line $lineNumber, cannot find tag/value" if $debug;
 			return undef;
 		}
 		
 	} else {
 
-		print "$fname: This line does not follow any known pattern: $line" if $debug;
+		print "This line does not follow any known pattern: $line" if $debug;
 	}
 
 	#If success
@@ -727,18 +688,21 @@ sub _analyseRadiusLine($$$) {
 }
 
 
-
 END {
 	
-        #Store computed hash tables
-        lock_store \%start, $startDbm or croak "Cannot store Start to file $startDbm: $!";
-        lock_store \%interim, $interimDbm or croak "Cannot store Interim to file $interimDbm: $!";
+	#Store computed Interim hash tables if not empty
+	if(scalar(keys %interim)){
+		lock_store \%interim, $interimDbm or croak "Cannot store Interim to file $interimDbm: $!";
+	}
 	
+	#Store computed Start hash tables if not empty
+	if(scalar(keys %start)){
+		lock_store \%start, $startDbm or croak "Cannot store Start to file $startDbm: $!";	
+	}
 }
 
 
 
-
 =head1 NAME
 
 RADIUS::XMLParser - Radius log file XML convertor
@@ -1002,3 +966,4 @@ modify it under the same terms as Perl itself.
 1;
 
 
+
